@@ -4,7 +4,7 @@ import {EditorView} from "./editorview"
 import {BlockType} from "./decoration"
 import {LineView} from "./blockview"
 import {atomicRanges} from "./extension"
-import {clientRectsFor, textRange, Rect} from "./dom"
+import {clientRectsFor, textRange, Rect, maxOffset} from "./dom"
 import {moveVisually, movedOver, Direction} from "./bidi"
 import {BlockInfo} from "./heightmap"
 import browser from "./browser"
@@ -71,8 +71,7 @@ function domPosAtCoords(parent: HTMLElement, x: number, y: number): {node: Node,
         return child.nodeType == 3 ? domPosInText(child as Text, x, y) : domPosAtCoords(child as HTMLElement, x, y)
       if (!closest || closestY > dy || closestY == dy && closestX > dx) {
         closest = child; closestRect = rect; closestX = dx; closestY = dy
-        let side = dy ? (y < rect.top ? -1 : 1) : dx ? (x < rect.left ? -1 : 1) : 0
-        closestOverlap = !side || (side > 0 ? i < rects.length - 1 : i > 0)
+        closestOverlap = !dx ? true : x < rect.left ? i > 0 : i < rects.length - 1
       }
       if (dx == 0) {
         if (y > rect.bottom && (!aboveRect || aboveRect.bottom < rect.bottom)) { above = child; aboveRect = rect }
@@ -184,6 +183,10 @@ export function posAtCoords(view: EditorView, coords: {x: number, y: number}, pr
           node = undefined
       }
     }
+    // Chrome will return offsets into <input> elements without child
+    // nodes, which will lead to a null deref below, so clip the
+    // offset to the node size.
+    if (node) offset = Math.min(maxOffset(node), offset)
   }
 
   // No luck, do our own (potentially expensive) search
@@ -219,11 +222,20 @@ function posAtCoordsImprecise(view: EditorView, contentRect: Rect, block: BlockI
 // line before. This is used to detect such a result so that it can be
 // ignored (issue #401).
 function isSuspiciousSafariCaretResult(node: Node, offset: number, x: number) {
-  let len
+  let len, scan = node
   if (node.nodeType != 3 || offset != (len = node.nodeValue!.length)) return false
-  for (let next = node.nextSibling; next; next = next.nextSibling)
-    if (next.nodeType != 1 || next.nodeName != "BR") return false
-  return textRange(node as Text, len - 1, len).getBoundingClientRect().left > x
+  for (;;) { // Check that there is no content after this node
+    let next = scan.nextSibling
+    if (next) {
+      if (next.nodeName == "BR") break
+      return false
+    } else {
+      let parent = scan.parentNode
+      if (!parent || parent.nodeName == "DIV") break
+      scan = parent
+    }
+  }
+  return textRange(node as Text, len - 1, len).getBoundingClientRect().right > x
 }
 
 // Chrome will move positions between lines to the start of the next line
@@ -240,16 +252,24 @@ function isSuspiciousChromeCaretResult(node: Node, offset: number, x: number) {
   return x - rect.left > 5
 }
 
-export function blockAt(view: EditorView, pos: number): BlockInfo {
+export function blockAt(view: EditorView, pos: number, side: -1 | 1): BlockInfo {
   let line = view.lineBlockAt(pos)
-  if (Array.isArray(line.type)) for (let l of line.type) {
-    if (l.to > pos || l.to == pos && (l.to == line.to || l.type == BlockType.Text)) return l
+  if (Array.isArray(line.type)) {
+    let best: BlockInfo | undefined
+    for (let l of line.type) {
+      if (l.from > pos) break
+      if (l.to < pos) continue
+      if (l.from < pos && l.to > pos) return l
+      if (!best || (l.type == BlockType.Text && (best.type != l.type || (side < 0 ? l.from < pos : l.to > pos))))
+        best = l
+    }
+    return best || line
   }
   return line
 }
 
 export function moveToLineBoundary(view: EditorView, start: SelectionRange, forward: boolean, includeWrap: boolean) {
-  let line = blockAt(view, start.head)
+  let line = blockAt(view, start.head, start.assoc || -1)
   let coords = !includeWrap || line.type != BlockType.Text || !(view.lineWrapping || line.widgetLineBreaks) ? null
     : view.coordsAtPos(start.assoc < 0 && start.head > line.from ? start.head - 1 : start.head)
   if (coords) {
