@@ -9,7 +9,7 @@ import {getAttrs} from "./attributes"
 import {clientRectsFor, isEquivalentPosition, Rect, scrollRectIntoView,
         getSelection, hasSelection, textRange, DOMSelectionState,
         textNodeBefore, textNodeAfter} from "./dom"
-import {ViewUpdate, decorations as decorationsFacet, outerDecorations, ChangedRange,
+import {ViewUpdate, decorations as decorationsFacet, outerDecorations, ChangedRange, editable,
         ScrollTarget, scrollHandler, getScrollMargins, logException, setEditContextFormatting} from "./extension"
 import {EditorView} from "./editorview"
 import {Direction} from "./bidi"
@@ -22,7 +22,7 @@ type Composition = {
 }
 
 export class DocView extends ContentView {
-  children!: BlockView[]
+  declare children: BlockView[]
 
   decorations: readonly DecorationSet[] = []
   dynamicDecorationMap: boolean[] = [false]
@@ -49,7 +49,7 @@ export class DocView extends ContentView {
   impreciseHead: DOMPos | null = null
   forceSelection = false
 
-  dom!: HTMLElement
+  declare dom: HTMLElement
 
   // Used by the resize observer to ignore resizes that we caused
   // ourselves
@@ -228,7 +228,7 @@ export class DocView extends ContentView {
   updateSelection(mustRead = false, fromPointer = false) {
     if (mustRead || !this.view.observer.selectionRange.focusNode) this.view.observer.readSelectionRange()
     let activeElt = this.view.root.activeElement, focused = activeElt == this.dom
-    let selectionNotFocus = !focused &&
+    let selectionNotFocus = !focused && !(this.view.state.facet(editable) || this.dom.tabIndex > -1) &&
       hasSelection(this.dom, this.view.observer.selectionRange) && !(activeElt && this.dom.contains(activeElt))
     if (!(focused || fromPointer || selectionNotFocus)) return
     let force = this.forceSelection
@@ -572,7 +572,7 @@ export class DocView extends ContentView {
   }
 
   // Will never be called but needs to be present
-  split!: () => ContentView
+  declare split: () => ContentView
 }
 
 function betweenUneditable(pos: DOMPos) {
@@ -644,6 +644,7 @@ class DecorationComparator {
   changes: number[] = []
   compareRange(from: number, to: number) { addRange(from, to, this.changes) }
   comparePoint(from: number, to: number) { addRange(from, to, this.changes) }
+  boundChange(pos: number) { addRange(pos, pos, this.changes) }
 }
 
 function findChangedDeco(a: readonly DecorationSet[], b: readonly DecorationSet[], diff: ChangeSet) {
