@@ -2,8 +2,7 @@ import browser from "./browser"
 import {ContentView, ViewFlag} from "./contentview"
 import {EditorView} from "./editorview"
 import {editable, ViewUpdate, setEditContextFormatting, MeasureRequest} from "./extension"
-import {hasSelection, getSelection, DOMSelectionState, isEquivalentPosition,
-        deepActiveElement, dispatchKey, atElementStart} from "./dom"
+import {hasSelection, getSelection, DOMSelectionState, isEquivalentPosition, dispatchKey, atElementStart} from "./dom"
 import {DOMChange, applyDOMChange, applyDOMChangeInner} from "./domchange"
 import type {EditContext} from "./editcontext"
 import {Decoration} from "./decoration"
@@ -81,7 +80,7 @@ export class DOMObserver {
         this.flush()
     })
 
-    if (window.EditContext && (view.constructor as any).EDIT_CONTEXT !== false &&
+    if (window.EditContext && browser.android && (view.constructor as any).EDIT_CONTEXT !== false &&
         // Chrome <126 doesn't support inverted selections in edit context (#1392)
         !(browser.chrome && browser.chrome_version < 126)) {
       this.editContext = new EditContextManager(view)
@@ -90,7 +89,7 @@ export class DOMObserver {
     }
 
     if (useCharData)
-      this.onCharData = (event: MutationEvent) => {
+      this.onCharData = (event: any) => {
         this.queue.push({target: event.target,
                          type: "characterData",
                          oldValue: event.prevValue} as MutationRecord)
@@ -172,7 +171,7 @@ export class DOMObserver {
     let wasChanged = this.selectionChanged
     if (!this.readSelectionRange() || this.delayedAndroidKey) return
     let {view} = this, sel = this.selectionRange
-    if (view.state.facet(editable) ? view.root.activeElement != this.dom : !hasSelection(view.dom, sel))
+    if (view.state.facet(editable) ? view.root.activeElement != this.dom : !hasSelection(this.dom, sel))
       return
 
     let context = sel.anchorNode && view.docView.nearest(sel.anchorNode)
@@ -201,7 +200,7 @@ export class DOMObserver {
     let selection = getSelection(view.root)
     if (!selection) return false
     let range = browser.safari && (view.root as any).nodeType == 11 &&
-      deepActiveElement(this.dom.ownerDocument) == this.dom &&
+      view.root.activeElement == this.dom &&
       safariSelectionRangeHack(this.view, selection) || selection
     if (!range || this.selectionRange.eq(range)) return false
     let local = hasSelection(this.dom, range)
@@ -541,6 +540,9 @@ class EditContextManager {
   // user action on some Android keyboards)
   pendingContextChange: {from: number, to: number, insert: Text} | null = null
   handlers: {[name: string]: (e: any) => void} = Object.create(null)
+  // Kludge to work around the fact that EditContext does not respond
+  // well to having its content updated during a composition (see #1472)
+  composing: {contextBase: number, editorBase: number, drifted: boolean} | null = null
 
   constructor(view: EditorView) {
     this.resetRange(view.state)
@@ -551,22 +553,32 @@ class EditContextManager {
       selectionEnd: this.toContextPos(view.state.selection.main.head)
     })
     this.handlers.textupdate = e => {
-      let {anchor} = view.state.selection.main
-      let change = {from: this.toEditorPos(e.updateRangeStart),
-                    to: this.toEditorPos(e.updateRangeEnd),
-                    insert: Text.of(e.text.split("\n"))}
+      let main = view.state.selection.main, {anchor, head} = main
+      let from = this.toEditorPos(e.updateRangeStart), to = this.toEditorPos(e.updateRangeEnd)
+      if (view.inputState.composing >= 0 && !this.composing)
+        this.composing = {contextBase: e.updateRangeStart, editorBase: from, drifted: false}
+      let change = {from, to, insert: Text.of(e.text.split("\n"))}
       // If the window doesn't include the anchor, assume changes
       // adjacent to a side go up to the anchor.
       if (change.from == this.from && anchor < this.from) change.from = anchor
       else if (change.to == this.to && anchor > this.to) change.to = anchor
 
       // Edit contexts sometimes fire empty changes
-      if (change.from == change.to && !change.insert.length) return
+      if (change.from == change.to && !change.insert.length) {
+        let newSel = EditorSelection.single(this.toEditorPos(e.selectionStart), this.toEditorPos(e.selectionEnd))
+        if (!newSel.main.eq(main)) view.dispatch({selection: newSel, userEvent: "select"})
+        return
+      }
+      if ((browser.mac || browser.android) && change.from == head - 1 &&
+          /^\. ?$/.test(e.text) && view.contentDOM.getAttribute("autocorrect") == "off")
+        change = {from, to, insert: Text.of([e.text.replace(".", " ")])}
 
       this.pendingContextChange = change
-      if (!view.state.readOnly)
-        applyDOMChangeInner(view, change, EditorSelection.single(this.toEditorPos(e.selectionStart),
-                                                                 this.toEditorPos(e.selectionEnd)))
+      if (!view.state.readOnly) {
+        let newLen = this.to - this.from + (change.to - change.from + change.insert.length)
+        applyDOMChangeInner(view, change, EditorSelection.single(this.toEditorPos(e.selectionStart, newLen),
+                                                                 this.toEditorPos(e.selectionEnd, newLen)))
+      }
       // If the transaction didn't flush our change, revert it so
       // that the context is in sync with the editor state again.
       if (this.pendingContextChange) {
@@ -589,11 +601,13 @@ class EditContextManager {
       for (let format of e.getTextFormats()) {
         let lineStyle = format.underlineStyle, thickness = format.underlineThickness
         if (lineStyle != "None" && thickness != "None") {
-          let style = `text-decoration: underline ${
-            lineStyle == "Dashed" ? "dashed " : lineStyle == "Squiggle" ? "wavy " : ""
-          }${thickness == "Thin" ? 1 : 2}px`
-          deco.push(Decoration.mark({attributes: {style}})
-            .range(this.toEditorPos(format.rangeStart), this.toEditorPos(format.rangeEnd)))
+          let from = this.toEditorPos(format.rangeStart), to = this.toEditorPos(format.rangeEnd)
+          if (from < to) {
+            let style = `text-decoration: underline ${
+              lineStyle == "Dashed" ? "dashed " : lineStyle == "Squiggle" ? "wavy " : ""
+            }${thickness == "Thin" ? 1 : 2}px`
+            deco.push(Decoration.mark({attributes: {style}}).range(from, to))
+          }
         }
       }
       view.dispatch({effects: setEditContextFormatting.of(Decoration.set(deco))})
@@ -607,6 +621,11 @@ class EditContextManager {
     this.handlers.compositionend = () => {
       view.inputState.composing = -1
       view.inputState.compositionFirstChange = null
+      if (this.composing) {
+        let {drifted} = this.composing
+        this.composing = null
+        if (drifted) this.reset(view.state)
+      }
     }
     for (let event in this.handlers) context.addEventListener(event as any, this.handlers[event])
 
@@ -654,12 +673,16 @@ class EditContextManager {
   }
 
   update(update: ViewUpdate) {
-    let reverted = this.pendingContextChange
-    if (!this.applyEdits(update) || !this.rangeIsValid(update.state)) {
+    let reverted = this.pendingContextChange, startSel = update.startState.selection.main
+    if (this.composing &&
+        (this.composing.drifted ||
+         (!update.changes.touchesRange(startSel.from, startSel.to) &&
+          update.transactions.some(tr => !tr.isUserEvent("input.type") && tr.changes.touchesRange(this.from, this.to))))) {
+      this.composing.drifted = true
+      this.composing.editorBase = update.changes.mapPos(this.composing.editorBase)
+    } else if (!this.applyEdits(update) || !this.rangeIsValid(update.state)) {
       this.pendingContextChange = null
-      this.resetRange(update.state)
-      this.editContext.updateText(0, this.editContext.text.length, update.state.doc.sliceString(this.from, this.to))
-      this.setSelection(update.state)
+      this.reset(update.state)
     } else if (update.docChanged || update.selectionSet || reverted) {
       this.setSelection(update.state)
     }
@@ -673,6 +696,12 @@ class EditContextManager {
     this.to = Math.min(state.doc.length, head + CxVp.Margin)
   }
 
+  reset(state: EditorState) {
+    this.resetRange(state)
+    this.editContext.updateText(0, this.editContext.text.length, state.doc.sliceString(this.from, this.to))
+    this.setSelection(state)
+  }
+
   revertPending(state: EditorState) {
     let pending = this.pendingContextChange!
     this.pendingContextChange = null
@@ -696,8 +725,15 @@ class EditContextManager {
              this.to - this.from > CxVp.Margin * 3)
   }
 
-  toEditorPos(contextPos: number) { return contextPos + this.from }
-  toContextPos(editorPos: number) { return editorPos - this.from }
+  toEditorPos(contextPos: number, clipLen = this.to - this.from) {
+    contextPos = Math.min(contextPos, clipLen)
+    let c = this.composing
+    return c && c.drifted ? c.editorBase + (contextPos - c.contextBase) : contextPos + this.from
+  }
+  toContextPos(editorPos: number) {
+    let c = this.composing
+    return c && c.drifted ? c.contextBase + (editorPos - c.editorBase) : editorPos - this.from
+  }
 
   destroy() {
     for (let event in this.handlers) this.editContext.removeEventListener(event as any, this.handlers[event])
