@@ -109,13 +109,13 @@ export class EditorView {
   /// Indicates whether the user is currently composing text via
   /// [IME](https://en.wikipedia.org/wiki/Input_method), and at least
   /// one change has been made in the current composition.
-  get composing() { return this.inputState.composing > 0 }
+  get composing() { return !!this.inputState && this.inputState.composing > 0 }
 
   /// Indicates whether the user is currently in composing state. Note
   /// that on some platforms, like Android, this will be the case a
   /// lot, since just putting the cursor on a word starts a
   /// composition there.
-  get compositionStarted() { return this.inputState.composing >= 0 }
+  get compositionStarted() { return !!this.inputState && this.inputState.composing >= 0 }
   
   private dispatchTransactions: (trs: readonly Transaction[], view: EditorView) => void
 
@@ -145,7 +145,7 @@ export class EditorView {
   private announceDOM: HTMLElement
 
   /// @internal
-  inputState!: InputState
+  declare inputState: InputState
 
   /// @internal
   public viewState: ViewState
@@ -153,10 +153,10 @@ export class EditorView {
   public docView: DocView
 
   private plugins: PluginInstance[] = []
-  private pluginMap: Map<ViewPlugin<any>, PluginInstance | null> = new Map
+  private pluginMap: Map<ViewPlugin<any, any>, PluginInstance | null> = new Map
   private editorAttrs: Attrs = {}
   private contentAttrs: Attrs = {}
-  private styleModules!: readonly StyleModule[]
+  declare private styleModules: readonly StyleModule[]
   private bidiCache: CachedOrder[] = []
 
   private destroyed = false;
@@ -517,6 +517,7 @@ export class EditorView {
       spellcheck: "false",
       autocorrect: "off",
       autocapitalize: "off",
+      writingsuggestions: "false",
       translate: "no",
       contenteditable: !this.state.facet(editable) ? "false" : "true",
       class: "cm-content",
@@ -584,10 +585,10 @@ export class EditorView {
   /// plugins that crash can be dropped from a view, so even when you
   /// know you registered a given plugin, it is recommended to check
   /// the return value of this method.
-  plugin<T extends PluginValue>(plugin: ViewPlugin<T>): T | null {
+  plugin<T extends PluginValue>(plugin: ViewPlugin<T, any>): T | null {
     let known = this.pluginMap.get(plugin)
-    if (known === undefined || known && known.spec != plugin)
-      this.pluginMap.set(plugin, known = this.plugins.find(p => p.spec == plugin) || null)
+    if (known === undefined || known && known.plugin != plugin)
+      this.pluginMap.set(plugin, known = this.plugins.find(p => p.plugin == plugin) || null)
     return known && known.update(this).value as T
   }
 
