@@ -9,8 +9,8 @@ import {EditorView} from "./editorview"
 const MaxJoinLen = 256
 
 export class TextView extends ContentView {
-  children!: ContentView[]
-  dom!: Text | null
+  declare children: ContentView[]
+  declare dom: Text | null
 
   constructor(public text: string) {
     super()
@@ -69,7 +69,7 @@ export class TextView extends ContentView {
 }
 
 export class MarkView extends ContentView {
-  dom!: HTMLElement | null
+  declare dom: HTMLElement | null
 
   constructor(readonly mark: MarkDecoration,
               public children: ContentView[] = [],
@@ -159,8 +159,8 @@ function textCoords(text: Text, pos: number, side: number): Rect | null {
 
 // Also used for collapsed ranges that don't have a placeholder widget!
 export class WidgetView extends ContentView {
-  children!: ContentView[]
-  dom!: HTMLElement | null
+  declare children: ContentView[]
+  declare dom: HTMLElement | null
   prevWidget: WidgetType | null = null
 
   static create(widget: WidgetType, length: number, side: number) {
@@ -256,8 +256,8 @@ export class WidgetView extends ContentView {
 // browser bugs that show up when the cursor is directly next to
 // uneditable inline content.
 export class WidgetBufferView extends ContentView {
-  children!: ContentView[]
-  dom!: HTMLElement | null
+  declare children: ContentView[]
+  declare dom: HTMLElement | null
 
   constructor(readonly side: number) { super() }
 
@@ -342,7 +342,7 @@ export function coordsInChildren(view: ContentView, pos: number, side: number):
       if (end >= pos) {
         if (child.children.length) {
           scan(child, pos - off)
-        } else if ((!after || after.isHidden && side > 0) &&
+        } else if ((!after || after.isHidden && (side > 0 || onSameLine(after, child))) &&
                    (end > pos || off == end && child.getSide() > 0)) {
           after = child
           afterPos = pos - off
@@ -366,3 +366,8 @@ function fallbackRect(view: ContentView) {
   let rects = clientRectsFor(last)
   return rects[rects.length - 1] || null
 }
+
+function onSameLine(a: ContentView, b: ContentView) {
+  let posA = a.coordsAt(0, 1), posB = b.coordsAt(0, 1)
+  return posA && posB && posB.top < posA.bottom
+}
