@@ -1,5 +1,5 @@
 import {EditorSelection, EditorState, SelectionRange, RangeSet, Annotation, Text, Facet} from "@codemirror/state"
-import {EditorView} from "./editorview"
+import {EditorView, UpdateState} from "./editorview"
 import {ContentView} from "./contentview"
 import {LineView} from "./blockview"
 import {ViewUpdate, PluginValue, clickAddsSelectionRange, dragMovesSelection as dragBehavior, atomicRanges,
@@ -10,7 +10,6 @@ import {groupAt, skipAtomicRanges} from "./cursor"
 import {getSelection, focusPreventScroll, Rect, dispatchKey, scrollableParents} from "./dom"
 import {applyDOMChangeInner} from "./domchange"
 
-// This will also be where dragging info and such goes
 export class InputState {
   lastKeyCode: number = 0
   lastKeyTime: number = 0
@@ -85,7 +84,8 @@ export class InputState {
   handleEvent(event: Event) {
     if (!eventBelongsToEditor(this.view, event) || this.ignoreDuringComposition(event)) return
     if (event.type == "keydown" && this.keydown(event as KeyboardEvent)) return
-    this.runHandlers(event.type, event)
+    if (this.view.updateState != UpdateState.Idle) Promise.resolve().then(() => this.runHandlers(event.type, event))
+    else this.runHandlers(event.type, event)
   }
 
   runHandlers(type: string, event: Event) {
@@ -218,13 +218,13 @@ function computeHandlers(plugins: readonly PluginInstance[]) {
     return result[type] || (result[type] = {observers: [], handlers: []})
   }
   for (let plugin of plugins) {
-    let spec = plugin.spec
-    if (spec && spec.domEventHandlers) for (let type in spec.domEventHandlers) {
-      let f = spec.domEventHandlers[type]
+    let spec = plugin.spec, handlers = spec && spec.plugin.domEventHandlers, observers = spec && spec.plugin.domEventObservers
+    if (handlers) for (let type in handlers) {
+      let f = handlers[type]
       if (f) record(type).handlers.push(bindHandler(plugin.value!, f))
     }
-    if (spec && spec.domEventObservers) for (let type in spec.domEventObservers) {
-      let f = spec.domEventObservers[type]
+    if (observers) for (let type in observers) {
+      let f = observers[type]
       if (f) record(type).observers.push(bindHandler(plugin.value!, f))
     }
   }
@@ -808,7 +808,7 @@ export function focusChangeTransaction(state: EditorState, focus: boolean) {
     let effect = getEffect(state, focus)
     if (effect) effects.push(effect)
   }
-  return effects ? state.update({effects, annotations: isFocusChange.of(true)}) : null
+  return effects.length ? state.update({effects, annotations: isFocusChange.of(true)}) : null
 }
 
 function updateForFocusChange(view: EditorView) {
