@@ -7,7 +7,7 @@ const plugin = ViewPlugin.fromClass(class {
 
   update(update: ViewUpdate) {
     let {view} = update
-    let height = view.viewState.editorHeight * view.scaleY -
+    let height = view.viewState.editorHeight -
       view.defaultLineHeight - view.documentPadding.top - 0.5
     if (height >= 0 && height != this.height) {
       this.height = height
