@@ -1,14 +1,14 @@
 import {EditorState, Transaction, StateEffect, StateEffectType,
         Facet, StateField, Extension, MapMode, FacetReader} from "@codemirror/state"
 import {EditorView} from "./editorview"
-import {ViewPlugin, ViewUpdate, logException} from "./extension"
+import {ViewPlugin, ViewUpdate, logException, getScrollMargins} from "./extension"
 import {Direction} from "./bidi"
 import {WidgetView} from "./inlineview"
 import {Rect} from "./dom"
 import browser from "./browser"
 
 type Measured = {
-  editor: DOMRect,
+  visible: Rect,
   parent: DOMRect,
   pos: (Rect | null)[],
   size: DOMRect[],
@@ -103,9 +103,10 @@ export function tooltips(config: {
   parent?: HTMLElement
   /// By default, when figuring out whether there is room for a
   /// tooltip at a given position, the extension considers the entire
-  /// space between 0,0 and `innerWidth`,`innerHeight` to be available
-  /// for showing tooltips. You can provide a function here that
-  /// returns an alternative rectangle.
+  /// space between 0,0 and
+  /// `documentElement.clientWidth`/`clientHeight` to be available for
+  /// showing tooltips. You can provide a function here that returns
+  /// an alternative rectangle.
   tooltipSpace?: (view: EditorView) => Rect
 } = {}): Extension {
   return tooltipConfig.of(config)
@@ -118,8 +119,8 @@ type TooltipConfig = {
 }
 
 function windowSpace(view: EditorView) {
-  let {win} = view
-  return {top: 0, left: 0, bottom: win.innerHeight, right: win.innerWidth}
+  let docElt = view.dom.ownerDocument.documentElement
+  return {top: 0, left: 0, bottom: docElt.clientHeight, right: docElt.clientWidth}
 }
 
 const tooltipConfig = Facet.define<Partial<TooltipConfig>, TooltipConfig>({
@@ -140,7 +141,7 @@ const tooltipPlugin = ViewPlugin.fromClass(class {
   position: "fixed" | "absolute"
   madeAbsolute = false
   parent: HTMLElement | null
-  container!: HTMLElement
+  declare container: HTMLElement
   classes: string
   intersectionObserver: IntersectionObserver | null
   resizeObserver: ResizeObserver | null
@@ -250,7 +251,6 @@ const tooltipPlugin = ViewPlugin.fromClass(class {
   }
 
   readMeasure(): Measured {
-    let editor = this.view.dom.getBoundingClientRect()
     let scaleX = 1, scaleY = 1, makeAbsolute = false
     if (this.position == "fixed" && this.manager.tooltipViews.length) {
       let {dom} = this.manager.tooltipViews[0]
@@ -277,9 +277,13 @@ const tooltipPlugin = ViewPlugin.fromClass(class {
         ;({scaleX, scaleY} = this.view.viewState)
       }
     }
+    let visible = this.view.scrollDOM.getBoundingClientRect(), margins = getScrollMargins(this.view)
     return {
-      editor,
-      parent: this.parent ? this.container.getBoundingClientRect() : editor,
+      visible: {
+        left: visible.left + margins.left, top: visible.top + margins.top,
+        right: visible.right - margins.right, bottom: visible.bottom - margins.bottom
+      },
+      parent: this.parent ? this.container.getBoundingClientRect() : this.view.dom.getBoundingClientRect(),
       pos: this.manager.tooltips.map((t, i) => {
         let tv = this.manager.tooltipViews[i]
         return tv.getCoords ? tv.getCoords(t.pos) : this.view.coordsAtPos(t.pos)
@@ -297,16 +301,17 @@ const tooltipPlugin = ViewPlugin.fromClass(class {
       for (let t of this.manager.tooltipViews) t.dom.style.position = "absolute"
     }
 
-    let {editor, space, scaleX, scaleY} = measured
+    let {visible, space, scaleX, scaleY} = measured
     let others = []
     for (let i = 0; i < this.manager.tooltips.length; i++) {
       let tooltip = this.manager.tooltips[i], tView = this.manager.tooltipViews[i], {dom} = tView
       let pos = measured.pos[i], size = measured.size[i]
       // Hide tooltips that are outside of the editor.
-      if (!pos || pos.bottom <= Math.max(editor.top, space.top) ||
-          pos.top >= Math.min(editor.bottom, space.bottom) ||
-          pos.right < Math.max(editor.left, space.left) - .1 ||
-          pos.left > Math.min(editor.right, space.right) + .1) {
+      if (!pos || tooltip.clip !== false && (
+            pos.bottom <= Math.max(visible.top, space.top) ||
+            pos.top >= Math.min(visible.bottom, space.bottom) ||
+            pos.right < Math.max(visible.left, space.left) - .1 ||
+            pos.left > Math.min(visible.right, space.right) + .1)) {
         dom.style.top = Outside
         continue
       }
@@ -320,8 +325,8 @@ const tooltipPlugin = ViewPlugin.fromClass(class {
         : Math.min(Math.max(space.left, pos.left - width + (arrow ? Arrow.Offset : 0) - offset.x), space.right - width)
       let above = this.above[i]
       if (!tooltip.strictSide && (above
-            ? pos.top - (size.bottom - size.top) - offset.y < space.top
-            : pos.bottom + (size.bottom - size.top) + offset.y > space.bottom) &&
+            ? pos.top - height - arrowHeight - offset.y < space.top
+            : pos.bottom + height + arrowHeight + offset.y > space.bottom) &&
           above == (space.bottom - pos.bottom > pos.top - space.top))
         above = this.above[i] = !above
       let spaceVert = (above ? pos.top - space.top : space.bottom - pos.bottom) - arrowHeight
@@ -339,10 +344,10 @@ const tooltipPlugin = ViewPlugin.fromClass(class {
           top = above ? r.top - height - 2 - arrowHeight : r.bottom + arrowHeight + 2
       if (this.position == "absolute") {
         dom.style.top = (top - measured.parent.top) / scaleY + "px"
-        dom.style.left = (left - measured.parent.left) / scaleX + "px"
+        setLeftStyle(dom, (left - measured.parent.left) / scaleX)
       } else {
         dom.style.top = top / scaleY + "px"
-        dom.style.left = left / scaleX + "px"
+        setLeftStyle(dom, left / scaleX)
       }
       if (arrow) {
         let arrowLeft = pos.left + (ltr ? offset.x : -offset.x) - (left + Arrow.Offset - Arrow.Size)
@@ -372,9 +377,14 @@ const tooltipPlugin = ViewPlugin.fromClass(class {
   }
 })
 
+function setLeftStyle(elt: HTMLElement, value: number) {
+  let current = parseInt(elt.style.left, 10)
+  if (isNaN(current) || Math.abs(value - current) > 1) elt.style.left = value + "px"
+}
+
 const baseTheme = EditorView.baseTheme({
   ".cm-tooltip": {
-    zIndex: 100,
+    zIndex: 500,
     boxSizing: "border-box"
   },
   "&light .cm-tooltip": {
@@ -459,6 +469,10 @@ export interface Tooltip {
   /// When set to true, show a triangle connecting the tooltip element
   /// to position `pos`.
   arrow?: boolean
+  /// By default, tooltips are hidden when their position is outside
+  /// of the visible editor content. Set this to false to turn that
+  /// off.
+  clip?: boolean
 }
 
 /// Describes the way a tooltip is displayed.
