@@ -37,6 +37,12 @@ function visiblePixelRange(dom: HTMLElement, paddingTop: number): Rect {
           top: top - (rect.top + paddingTop), bottom: Math.max(top, bottom) - (rect.top + paddingTop)}
 }
 
+function inWindow(elt: HTMLElement) {
+  let rect = elt.getBoundingClientRect(), win = elt.ownerDocument.defaultView || window
+  return rect.left < win.innerWidth && rect.right > 0 &&
+    rect.top < win.innerHeight && rect.bottom > 0
+}
+
 function fullPixelRange(dom: HTMLElement, paddingTop: number): Rect {
   let rect = dom.getBoundingClientRect()
   return {left: 0, right: rect.right - rect.left,
@@ -51,14 +57,15 @@ const enum VP {
   MaxCoverMargin = VP.Margin / 4,
   // Beyond this size, DOM layout starts to break down in browsers
   // because they use fixed-precision numbers to store dimensions.
-  MaxDOMHeight = 7e6
+  MaxDOMHeight = 7e6,
+  MaxHorizGap = 2e6
 }
 
 // Line gaps are placeholder widgets used to hide pieces of overlong
 // lines within the viewport, as a kludge to keep the editor
 // responsive when a ridiculously long line is loaded into it.
 export class LineGap {
-  constructor(readonly from: number, readonly to: number, readonly size: number) {}
+  constructor(readonly from: number, readonly to: number, readonly size: number, readonly displaySize: number) {}
 
   static same(a: readonly LineGap[], b: readonly LineGap[]) {
     if (a.length != b.length) return false
@@ -71,7 +78,7 @@ export class LineGap {
 
   draw(viewState: ViewState, wrapping: boolean) {
     return Decoration.replace({
-      widget: new LineGapWidget(this.size * (wrapping ? viewState.scaleY : viewState.scaleX), wrapping)
+      widget: new LineGapWidget(this.displaySize * (wrapping ? viewState.scaleY : viewState.scaleX), wrapping)
     }).range(this.from, this.to)
   }
 }
@@ -140,15 +147,15 @@ export class ViewState {
   mustMeasureContent = true
 
   stateDeco: readonly DecorationSet[]
-  viewportLines!: BlockInfo[]
+  declare viewportLines: BlockInfo[]
   defaultTextDirection: Direction = Direction.LTR
 
   // The main viewport for the visible part of the document
-  viewport!: Viewport
+  declare viewport: Viewport
   // If the main selection starts or ends outside of the main
   // viewport, extra single-line viewports are created for these
   // points, so that the DOM selection doesn't fall in a gap.
-  viewports!: readonly Viewport[]
+  declare viewports: readonly Viewport[]
   visibleRanges: readonly {from: number, to: number}[] = []
   lineGaps: readonly LineGap[]
   lineGapDeco: DecorationSet
@@ -227,13 +234,14 @@ export class ViewState {
       this.scrollAnchorHeight = scrollAnchor.top
     } else {
       this.scrollAnchorPos = -1
-      this.scrollAnchorHeight = this.heightMap.height
+      this.scrollAnchorHeight = prevHeight
     }
 
     let viewport = heightChanges.length ? this.mapViewport(this.viewport, update.changes) : this.viewport
     if (scrollTarget && (scrollTarget.range.head < viewport.from || scrollTarget.range.head > viewport.to) ||
         !this.viewportIsAppropriate(viewport))
       viewport = this.getViewport(0, scrollTarget)
+
     let viewportChange = viewport.from != this.viewport.from || viewport.to != this.viewport.to
     this.viewport = viewport
     update.flags |= this.updateForViewport()
@@ -241,7 +249,7 @@ export class ViewState {
 
     if (this.lineGaps.length || this.viewport.to - this.viewport.from > (LG.Margin << 1))
       this.updateLineGaps(this.ensureLineGaps(this.mapLineGaps(this.lineGaps, update.changes)))
-    update.flags |= this.computeVisibleRanges()
+    update.flags |= this.computeVisibleRanges(update.changes)
 
     if (scrollTarget) this.scrollTarget = scrollTarget
 
@@ -303,7 +311,7 @@ export class ViewState {
       this.inView = inView
       if (inView) measureContent = true
     }
-    if (!this.inView && !this.scrollTarget) return 0
+    if (!this.inView && !this.scrollTarget && !inWindow(view.dom)) return 0
 
     let contentWidth = domRect.width
     if (this.contentDOMWidth != contentWidth || this.editorHeight != view.scrollDOM.clientHeight) {
@@ -318,7 +326,7 @@ export class ViewState {
       if (refresh || oracle.lineWrapping && Math.abs(contentWidth - this.contentDOMWidth) > oracle.charWidth) {
         let {lineHeight, charWidth, textHeight} = view.docView.measureTextSize()
         refresh = lineHeight > 0 && oracle.refresh(whiteSpace, lineHeight, charWidth, textHeight,
-                                                   contentWidth / charWidth, lineHeights)
+                                                   Math.max(5, contentWidth / charWidth), lineHeights)
         if (refresh) {
           view.docView.minWidth = 0
           result |= UpdateFlag.Geometry
@@ -420,7 +428,7 @@ export class ViewState {
     if (!gaps.length || changes.empty) return gaps
     let mapped = []
     for (let gap of gaps) if (!changes.touchesRange(gap.from, gap.to))
-      mapped.push(new LineGap(changes.mapPos(gap.from), changes.mapPos(gap.to), gap.size))
+      mapped.push(new LineGap(changes.mapPos(gap.from), changes.mapPos(gap.to), gap.size, gap.displaySize))
     return mapped
   }
 
@@ -458,7 +466,9 @@ export class ViewState {
           let lineStart = mayMeasure.moveToLineBoundary(EditorSelection.cursor(to), false, true).head
           if (lineStart > from) to = lineStart
         }
-        gap = new LineGap(from, to, this.gapSize(line, from, to, structure))
+        let size = this.gapSize(line, from, to, structure)
+        let displaySize = wrapping || size < VP.MaxHorizGap ? size : VP.MaxHorizGap
+        gap = new LineGap(from, to, size, displaySize)
       }
       gaps.push(gap)
     }
@@ -486,15 +496,22 @@ export class ViewState {
       } else {
         let totalWidth = structure.total * this.heightOracle.charWidth
         let marginWidth = margin * this.heightOracle.charWidth
+        let horizOffset = 0
+        if (totalWidth > VP.MaxHorizGap) for (let old of current) {
+          if (old.from >= line.from && old.from < line.to && old.size != old.displaySize &&
+              old.from * this.heightOracle.charWidth + horizOffset < this.pixelViewport.left)
+            horizOffset = old.size - old.displaySize
+        }
+        let pxLeft = this.pixelViewport.left + horizOffset, pxRight = this.pixelViewport.right + horizOffset
         let left, right
         if (target != null) {
           let targetFrac = findFraction(structure, target)
-          let spaceFrac = ((this.pixelViewport.right - this.pixelViewport.left) / 2 + marginWidth) / totalWidth
+          let spaceFrac = ((pxRight - pxLeft) / 2 + marginWidth) / totalWidth
           left = targetFrac - spaceFrac
           right = targetFrac + spaceFrac
         } else {
-          left = (this.pixelViewport.left - marginWidth) / totalWidth
-          right = (this.pixelViewport.right + marginWidth) / totalWidth
+          left = (pxLeft - marginWidth) / totalWidth
+          right = (pxRight + marginWidth) / totalWidth
         }
         viewFrom = findPosition(structure, left)
         viewTo = findPosition(structure, right)
@@ -527,7 +544,7 @@ export class ViewState {
     }
   }
 
-  computeVisibleRanges() {
+  computeVisibleRanges(changes?: ChangeDesc) {
     let deco = this.stateDeco
     if (this.lineGaps.length) deco = deco.concat(this.lineGapDeco)
     let ranges: {from: number, to: number}[] = []
@@ -535,10 +552,21 @@ export class ViewState {
       span(from, to) { ranges.push({from, to}) },
       point() {}
     }, 20)
-    let changed = ranges.length != this.visibleRanges.length ||
-      this.visibleRanges.some((r, i) => r.from != ranges[i].from || r.to != ranges[i].to)
+    let changed = 0
+    if (ranges.length != this.visibleRanges.length) {
+      changed = UpdateFlag.ViewportMoved | UpdateFlag.Viewport
+    } else {
+      for (let i = 0; i < ranges.length && !(changed & UpdateFlag.ViewportMoved); i++) {
+        let old = this.visibleRanges[i], nw = ranges[i]
+        if (old.from != nw.from || old.to != nw.to) {
+          changed |= UpdateFlag.Viewport
+          if (!(changes && changes.mapPos(old.from, -1) == nw.from && changes.mapPos(old.to, 1) == nw.to))
+            changed |= UpdateFlag.ViewportMoved
+        }
+      }
+    }
     this.visibleRanges = ranges
-    return changed ? UpdateFlag.Viewport : 0
+    return changed
   }
 
   lineBlockAt(pos: number): BlockInfo {
